@@ -8,15 +8,12 @@
 #include "CaptureWindow.h"
 
 #include "Config.h"
+#include "Gui/SpectacleWindow.h"
 #include "SpectacleCore.h"
 #include "Gui/SelectionEditor.h"
+#include "spectacle_gui_debug.h"
 
-#include <QScreen>
-#include <QShortcut>
-
-using namespace Qt::StringLiterals;
-
-QList<CaptureWindow *> CaptureWindow::s_captureWindowInstances = {};
+QVector<CaptureWindow *> CaptureWindow::s_captureWindowInstances = {};
 
 CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWindow *parent)
     : SpectacleWindow(engine, parent)
@@ -25,14 +22,6 @@ CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWi
     s_captureWindowInstances.append(this);
     s_isAnnotating = true;
 
-    // Set up shortcuts. We won't need to access these again and the memory will be managed by Qt.
-    new QShortcut(QKeySequence::Save, this, this, &CaptureWindow::save);
-    new QShortcut(QKeySequence::SaveAs, this, this, &CaptureWindow::saveAs);
-    new QShortcut(QKeySequence::Copy, this, this, &CaptureWindow::copyImage);
-    new QShortcut(QKeySequence::Print, this, this, &CaptureWindow::showPrintDialog);
-
-    m_context->setContextObject(this); // Must be before QML is initialized
-
     setFlags({
         Qt::Window, // the default window flag
         Qt::FramelessWindowHint,
@@ -44,12 +33,31 @@ CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWi
 
     this->setColor(Qt::transparent);
 
+    // setup selectionEditor
+    auto selectionEditor = SelectionEditor::instance();
+    connect(selectionEditor, &SelectionEditor::screensRectChanged, this, [this]() {
+        syncGeometryWithScreen();
+    });
+    connect(selectionEditor, &SelectionEditor::screenImagesChanged, this, &CaptureWindow::screenCaptureUrlChanged);
+    connect(this, &CaptureWindow::devicePixelRatioChanged,
+            selectionEditor, &SelectionEditor::devicePixelRatioChanged);
+
+    // set up QML
+    setMode(mode); // sets source and other stuff based on mode.
+    if (auto rootItem = rootObject()) {
+        rootItem->installEventFilter(selectionEditor);
+    }
+
     // follow a screen
     connect(screen, &QScreen::geometryChanged, this, &CaptureWindow::syncGeometryWithScreen);
-    connect(screen, &QScreen::physicalDotsPerInchChanged,
-            this, &CaptureWindow::syncGeometryWithScreen);
+    connect(screen, &QScreen::physicalDotsPerInchChanged, this, [this]() {
+        Q_EMIT devicePixelRatioChanged(m_screenToFollow->devicePixelRatio());
+        syncGeometryWithScreen();
+    });
     syncGeometryWithScreen();
     Q_EMIT screenToFollowChanged();
+    Q_EMIT devicePixelRatioChanged(m_screenToFollow->devicePixelRatio());
+    Q_EMIT screenCaptureUrlChanged();
 
     // sync visibility
     connect(this, &QWindow::visibilityChanged, this, [this](QWindow::Visibility visibility){
@@ -65,18 +73,6 @@ CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWi
         }
         s_synchronizingVisibility = false;
     });
-
-    // setup selectionEditor
-    auto selectionEditor = SelectionEditor::instance();
-    connect(selectionEditor, &SelectionEditor::screensRectChanged, this, [this]() {
-        syncGeometryWithScreen();
-    });
-
-    // set up QML
-    setMode(mode); // sets source and other stuff based on mode.
-    if (auto rootItem = rootObject()) {
-        rootItem->installEventFilter(selectionEditor);
-    }
 }
 
 CaptureWindow::~CaptureWindow()
@@ -89,13 +85,11 @@ CaptureWindow::~CaptureWindow()
 
 CaptureWindow::UniquePointer CaptureWindow::makeUnique(Mode mode, QScreen *screen, QQmlEngine *engine, QWindow *parent)
 {
-    return UniquePointer(new CaptureWindow(mode, screen, engine, parent), [](CaptureWindow *window){
-        s_captureWindowInstances.removeOne(window);
-        deleter(window);
-    });
+    return UniquePointer(new CaptureWindow(mode, screen, engine, parent),
+                         &SpectacleWindow::deleter);
 }
 
-QList<CaptureWindow *> CaptureWindow::instances()
+QVector<CaptureWindow *> CaptureWindow::instances()
 {
     return s_captureWindowInstances;
 }
@@ -105,16 +99,28 @@ QScreen *CaptureWindow::screenToFollow() const
     return m_screenToFollow;
 }
 
+QString CaptureWindow::screenCaptureUrl() const
+{
+    if (!m_screenToFollow) {
+        return QString();
+    }
+    return QStringLiteral("image://spectacle/screen/") + m_screenToFollow->name() + QLatin1Char('/')
+        + QString::number(SelectionEditor::instance()->imageForScreen(m_screenToFollow).cacheKey());
+}
+
 void CaptureWindow::setMode(CaptureWindow::Mode mode)
 {
-    syncGeometryWithScreen();
-    QVariantMap initialProperties = {
-        // Set the parent in initialProperties to avoid having
-        // the parent and window be null in Component.onCompleted
-        {u"parent"_s, QVariant::fromValue(contentItem())}
-    };
-    setSource(QUrl("%1/Gui/CaptureOverlay.qml"_L1.arg(SPECTACLE_QML_PATH)),
-              initialProperties);
+    if (mode == Image) {
+        syncGeometryWithScreen();
+        QVariantMap initialProperties = {
+            // Set the parent in initialProperties to avoid having
+            // the parent and window be null in Component.onCompleted
+            {QStringLiteral("parent"), QVariant::fromValue(contentItem())}
+        };
+        setSource(QUrl(QStringLiteral("qrc:/src/Gui/ImageCaptureOverlay.qml")), initialProperties);
+    } else if (mode == Video) {
+        
+    }
 }
 
 bool CaptureWindow::accept()
@@ -124,22 +130,26 @@ bool CaptureWindow::accept()
 
 void CaptureWindow::save()
 {
-    SelectionEditor::instance()->acceptSelection(ExportManager::Save | ExportManager::UserAction);
+    accept();
+    SpectacleWindow::save();
 }
 
 void CaptureWindow::saveAs()
 {
-    SelectionEditor::instance()->acceptSelection(ExportManager::SaveAs | ExportManager::UserAction);
+    accept();
+    SpectacleWindow::saveAs();
 }
 
 void CaptureWindow::copyImage()
 {
-    SelectionEditor::instance()->acceptSelection(ExportManager::CopyImage | ExportManager::UserAction);
+    accept();
+    SpectacleWindow::copyImage();
 }
 
 void CaptureWindow::copyLocation()
 {
-    SelectionEditor::instance()->acceptSelection(ExportManager::Save | ExportManager::CopyPath | ExportManager::UserAction);
+    accept();
+    SpectacleWindow::copyLocation();
 }
 
 void CaptureWindow::mousePressEvent(QMouseEvent *event)
@@ -148,6 +158,37 @@ void CaptureWindow::mousePressEvent(QMouseEvent *event)
     SpectacleWindow::mousePressEvent(event);
 }
 
+void CaptureWindow::keyReleaseEvent(QKeyEvent *event)
+{
+    SpectacleWindow::keyReleaseEvent(event);
+    if (event->isAccepted()) {
+        return;
+    }
+    if (event->matches(QKeySequence::Save)) {
+        event->accept();
+        save();
+    } else if (event->matches(QKeySequence::SaveAs)) {
+        event->accept();
+        saveAs();
+    } else if (event->matches(QKeySequence::Copy)) {
+        event->accept();
+        copyImage();
+    } else if (event->matches(QKeySequence::Print)) {
+        event->accept();
+        showPrintDialog();
+    }
+    auto document = SpectacleCore::instance()->annotationDocument();
+    if (!event->isAccepted() && document) {
+        if (document->undoStackDepth() > 0 && event->matches(QKeySequence::Undo)) {
+            event->accept();
+            document->undo();
+        } else if (document->redoStackDepth() > 0 && event->matches(QKeySequence::Redo)) {
+            event->accept();
+            document->redo();
+        }
+    }
+}
+
 void CaptureWindow::showEvent(QShowEvent *event)
 {
     SpectacleWindow::showEvent(event);
@@ -169,5 +210,3 @@ void CaptureWindow::syncGeometryWithScreen()
     setGeometry(screenRect);
     setScreen(m_screenToFollow);
 }
-
-#include "moc_CaptureWindow.cpp"
