@@ -13,6 +13,7 @@
 #include "settings.h"
 
 #include <QFontDatabase>
+#include <QWindow>
 
 #include <KLocalizedString>
 #include <KShortcutWidget>
@@ -37,6 +38,14 @@ QSize SettingsDialog::sizeHint() const
     return QSize(qRound(58 * fontSize), qRound(62 * fontSize));
 }
 
+void SettingsDialog::showEvent(QShowEvent *event)
+{
+    auto parent = parentWidget();
+    bool onTop = parent && parent->windowHandle()->flags().testFlag(Qt::WindowStaysOnTopHint);
+    windowHandle()->setFlag(Qt::WindowStaysOnTopHint, onTop);
+    KConfigDialog::showEvent(event);
+}
+
 bool SettingsDialog::hasChanged()
 {
     return mShortcutsPage->isModified() || KConfigDialog::hasChanged();
