@@ -8,24 +8,32 @@
 
 #include "SpectacleWindow.h"
 
-#include "ExportManager.h"
 #include "SpectacleCore.h"
-#include "Geometry.h"
-#include "Gui/ExportMenu.h"
-#include "Gui/HelpMenu.h"
-#include "Gui/OptionsMenu.h"
-#include "Gui/WidgetWindowUtils.h"
+#include "spectacle_gui_debug.h"
 
+#include <KIO/JobUiDelegateFactory>
+#include <KIO/OpenFileManagerWindowJob>
+#include <KIO/OpenUrlJob>
+#include <KUrlMimeData>
+#include <KWindowSystem>
+
+#include <Kirigami/Units>
 #include <QApplication>
 #include <QColorDialog>
+#include <QDrag>
 #include <QFontDialog>
 #include <QtQml>
 #include <utility>
 
-using namespace Qt::StringLiterals;
-using G = Geometry;
+#ifdef XCB_FOUND
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+#include <QX11Info>
+#else
+#include <private/qtx11extras_p.h>
+#endif
+#endif
 
-QList<SpectacleWindow *> SpectacleWindow::s_spectacleWindowInstances = {};
+QVector<SpectacleWindow *> SpectacleWindow::s_spectacleWindowInstances = {};
 bool SpectacleWindow::s_synchronizingVisibility = false;
 bool SpectacleWindow::s_synchronizingTitle = false;
 SpectacleWindow::TitlePreset SpectacleWindow::s_lastTitlePreset = Default;
@@ -35,24 +43,57 @@ bool SpectacleWindow::s_isAnnotating = false;
 
 SpectacleWindow::SpectacleWindow(QQmlEngine *engine, QWindow *parent)
     : QQuickView(engine, parent)
+    , m_exportMenu(new ExportMenu)
+    , m_optionsMenu(new OptionsMenu)
+    , m_helpMenu(new HelpMenu)
     , m_context(new QQmlContext(engine->rootContext(), this))
 {
     s_spectacleWindowInstances.append(this);
 
+    if (m_exportMenu->winId()) {
+        m_exportMenu->windowHandle()->setTransientParent(this);
+    }
+    if (m_optionsMenu->winId()) {
+        m_optionsMenu->windowHandle()->setTransientParent(this);
+    }
+    if (m_helpMenu->winId()) {
+        m_helpMenu->windowHandle()->setTransientParent(this);
+    }
     connect(engine, &QQmlEngine::quit, QCoreApplication::instance(), &QCoreApplication::quit, Qt::QueuedConnection);
     connect(this, &QQuickView::statusChanged, this, [](QQuickView::Status status){
         if (status == QQuickView::Error) {
             QCoreApplication::quit();
         }
     });
-    connect(this, &SpectacleWindow::xChanged, this, &SpectacleWindow::logicalXChanged);
-    connect(this, &SpectacleWindow::yChanged, this, &SpectacleWindow::logicalYChanged);
+
+    // before we do anything, we need to set a window property
+    // that skips the close/hide window animation on kwin. this
+    // fixes a ghost image of the spectacle window that appears
+    // on subsequent screenshots taken with the take new screenshot
+    // button
+    //
+    // credits for this goes to Thomas Lübking <thomas.luebking@gmail.com>
+
+#ifdef XCB_FOUND
+    if (KWindowSystem::isPlatformX11()) {
+        // do the xcb shenanigans
+        xcb_connection_t *xcbConn = QX11Info::connection();
+        const QByteArray effectName = QByteArrayLiteral("_KDE_NET_WM_SKIP_CLOSE_ANIMATION");
+
+        xcb_intern_atom_cookie_t atomCookie = xcb_intern_atom_unchecked(xcbConn, false, effectName.length(), effectName.constData());
+        QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atom(xcb_intern_atom_reply(xcbConn, atomCookie, nullptr));
+        if (!atom.isNull()) {
+            uint32_t value = 1;
+            xcb_change_property(xcbConn, XCB_PROP_MODE_REPLACE, winId(), atom->atom, XCB_ATOM_CARDINAL, 32, 1, &value);
+        }
+    }
+#endif
 
     setTextRenderType(QQuickWindow::NativeTextRendering);
 
     // set up QML
     setResizeMode(QQuickView::SizeRootObjectToView);
-    m_context->setContextProperty(u"contextWindow"_s, this);
+    m_context->setContextProperty(QStringLiteral("contextWindow"), this);
 }
 
 SpectacleWindow::~SpectacleWindow()
@@ -60,14 +101,19 @@ SpectacleWindow::~SpectacleWindow()
     s_spectacleWindowInstances.removeOne(this);
 }
 
-qreal SpectacleWindow::logicalX() const
+ExportMenu *SpectacleWindow::exportMenu() const
 {
-    return G::mapFromPlatformValue(x(), devicePixelRatio());
+    return m_exportMenu.get();
 }
 
-qreal SpectacleWindow::logicalY() const
+OptionsMenu *SpectacleWindow::optionsMenu() const
 {
-    return G::mapFromPlatformValue(y(), devicePixelRatio());
+    return m_optionsMenu.get();
+}
+
+HelpMenu *SpectacleWindow::helpMenu() const
+{
+    return m_helpMenu.get();
 }
 
 bool SpectacleWindow::isAnnotating() const
@@ -94,7 +140,7 @@ void SpectacleWindow::unminimize()
     setWindowStates(windowStates().setFlag(Qt::WindowMinimized, false));
 }
 
-QList<SpectacleWindow *> SpectacleWindow::instances()
+QVector<SpectacleWindow *> SpectacleWindow::instances()
 {
     return s_spectacleWindowInstances;
 }
@@ -144,22 +190,7 @@ void SpectacleWindow::closeAll()
 
 qreal SpectacleWindow::dprRound(qreal value) const
 {
-    return G::dprRound(value, devicePixelRatio());
-}
-
-QPointF SpectacleWindow::dprRound(const QPointF &point) const
-{
-    return G::dprRound(point, devicePixelRatio());
-}
-
-qreal SpectacleWindow::dprCeil(qreal value) const
-{
-    return G::dprCeil(value, devicePixelRatio());
-}
-
-qreal SpectacleWindow::dprFloor(qreal value) const
-{
-    return G::dprFloor(value, devicePixelRatio());
+    return std::round(value * devicePixelRatio()) / devicePixelRatio();
 }
 
 QString SpectacleWindow::baseFileName(const QUrl &url) const
@@ -173,11 +204,11 @@ QString SpectacleWindow::titlePresetString(TitlePreset preset, const QString &fi
         return i18ncp("@title:window", "%1 second", "%1 seconds",
                       qCeil(SpectacleCore::instance()->captureTimeRemaining() / 1000.0));
     } else if (preset == TitlePreset::Unsaved) {
-        return i18nc("@title:window Unsaved Screenshot", "Unsaved") + u"*"_s;
+        return i18nc("@title:window Unsaved Screenshot", "Unsaved") + QStringLiteral("*");
     } else if (preset == TitlePreset::Saved && !fileName.isEmpty()) {
         return fileName;
     } else if (preset == TitlePreset::Modified && !fileName.isEmpty()) {
-        return fileName + u"*"_s;
+        return fileName + QStringLiteral("*");
     } else if (preset == TitlePreset::Previous && !s_previousTitle.isEmpty()) {
         return s_previousTitle;
     }
@@ -186,7 +217,6 @@ QString SpectacleWindow::titlePresetString(TitlePreset preset, const QString &fi
 
 void SpectacleWindow::deleter(SpectacleWindow *window)
 {
-    s_spectacleWindowInstances.removeOne(window);
     window->deleteLater();
 }
 
@@ -230,67 +260,73 @@ void SpectacleWindow::setSource(const QUrl &source, const QVariantMap &initialPr
 
 void SpectacleWindow::save()
 {
-    SpectacleCore::instance()->syncExportImage();
-    ExportManager::instance()->exportImage(ExportManager::Save | ExportManager::UserAction,
-                                           SpectacleCore::instance()->outputUrl());
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
+    // emits ExportManager::forceNotify when quitChecked is true,
+    // which is connected to SpectacleCore::doNotify,
+    // which emits SpectacleCore::allDone,
+    // which is connected to QCoreApplication::quit in Main.cpp
+    ExportManager::instance()->doSave(QUrl(), /* notify */ quitChecked);
+    if (quitChecked) {
+        SpectacleWindow::closeAll();
+    }
 }
 
 void SpectacleWindow::saveAs()
 {
-    if (SpectacleCore::instance()->videoMode()) {
-        ExportManager::instance()->exportVideo(ExportManager::SaveAs | ExportManager::UserAction,
-                                               SpectacleCore::instance()->currentVideo());
-        return;
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
+    if (ExportManager::instance()->doSaveAs(/* notify */ quitChecked) && quitChecked) {
+        SpectacleWindow::closeAll();
     }
-    SpectacleCore::instance()->syncExportImage();
-    ExportManager::instance()->exportImage(ExportManager::SaveAs | ExportManager::UserAction);
 }
 
 void SpectacleWindow::copyImage()
 {
-    SpectacleCore::instance()->syncExportImage();
-    ExportManager::instance()->exportImage(ExportManager::CopyImage | ExportManager::UserAction);
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
+    SpectacleCore::instance()->syncExportPixmap();
+    ExportManager::instance()->doCopyToClipboard(/* notify */ quitChecked);
+    if (quitChecked) {
+        SpectacleWindow::closeAll();
+    }
 }
 
 void SpectacleWindow::copyLocation()
 {
-    if (SpectacleCore::instance()->videoMode()) {
-        ExportManager::instance()->exportVideo(ExportManager::CopyPath | ExportManager::UserAction,
-                                               SpectacleCore::instance()->currentVideo());
-        return;
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
+    ExportManager::instance()->doCopyLocationToClipboard();
+    if (quitChecked) {
+        SpectacleWindow::closeAll();
     }
-    SpectacleCore::instance()->syncExportImage();
-    ExportManager::instance()->exportImage(ExportManager::CopyPath | ExportManager::UserAction);
 }
 
 void SpectacleWindow::showPrintDialog()
 {
-    SpectacleCore::instance()->syncExportImage();
-    ExportMenu::instance()->openPrintDialog();
+    m_exportMenu->openPrintDialog();
 }
 
 void SpectacleWindow::showPreferencesDialog()
 {
-    OptionsMenu::instance()->showPreferencesDialog();
+    m_optionsMenu->showPreferencesDialog();
 }
 
 void SpectacleWindow::showFontDialog()
 {
     auto tool = SpectacleCore::instance()->annotationDocument()->tool();
-    auto wrapper = SpectacleCore::instance()->annotationDocument()->selectedItemWrapper();
+    auto saWrapper = SpectacleCore::instance()->annotationDocument()->selectedActionWrapper();
     QFont font;
-    if (tool->type() == AnnotationTool::SelectTool
-        || (tool->type() == AnnotationTool::TextTool
-            && wrapper->options().testFlag(AnnotationTool::TextOption))
+    if (tool->type() == AnnotationDocument::ChangeAction
+        || (tool->type() == AnnotationDocument::Text
+            && saWrapper->type() == AnnotationDocument::Text)
     ) {
-        font = wrapper->font();
+        font = saWrapper->font();
     } else {
         font = tool->font();
     }
     QFontDialog *dialog = new QFontDialog(font);
     dialog->setAttribute(Qt::WA_DeleteOnClose);
 
-    setWidgetTransientParent(dialog, this);
+    if (dialog->winId()) {
+        dialog->windowHandle()->setTransientParent(this);
+    }
 
     if (flags().testFlag(Qt::WindowStaysOnTopHint)) {
         dialog->setWindowFlag(Qt::WindowStaysOnTopHint);
@@ -303,72 +339,68 @@ void SpectacleWindow::showFontDialog()
         // https://bugreports.qt.io/browse/QTBUG-63792
         // https://bugs.kde.org/show_bug.cgi?id=378523
         if (newFont.weight() == QFont::Normal
-            && (newFont.styleName() == "Regular"_L1
-                || newFont.styleName() == "Normal"_L1
-                || newFont.styleName() == "Book"_L1
-                || newFont.styleName() == "Roman"_L1)) {
+            && (newFont.styleName() == QLatin1String("Regular")
+                || newFont.styleName() == QLatin1String("Normal")
+                || newFont.styleName() == QLatin1String("Book")
+                || newFont.styleName() == QLatin1String("Roman"))) {
             newFont.setStyleName(QString());
         }
         auto tool = SpectacleCore::instance()->annotationDocument()->tool();
-        auto wrapper = SpectacleCore::instance()->annotationDocument()->selectedItemWrapper();
-        if (tool->type() == AnnotationTool::SelectTool) {
-            wrapper->setFont(newFont);
-            wrapper->commitChanges();
-        } else if (tool->type() == AnnotationTool::TextTool
-            && wrapper->options().testFlag(AnnotationTool::TextOption)
+        auto saWrapper = SpectacleCore::instance()->annotationDocument()->selectedActionWrapper();
+        if (tool->type() == AnnotationDocument::ChangeAction) {
+            saWrapper->setFont(newFont);
+            saWrapper->commitChanges();
+        } else if (tool->type() == AnnotationDocument::Text
+            && saWrapper->type() == AnnotationDocument::Text
         ) {
             tool->setFont(newFont);
-            wrapper->setFont(newFont);
-            wrapper->commitChanges();
+            saWrapper->setFont(newFont);
+            saWrapper->commitChanges();
         } else {
             tool->setFont(newFont);
         }
     });
 
-    // BUG https://bugs.kde.org/show_bug.cgi?id=478155:
-    // Workaround modal font dialog being unusable.
-    // This should probably be fixed in the plasma-integration.
-    dialog->setModal(false);
-    dialog->show();
+    dialog->open();
 }
 
 void SpectacleWindow::showColorDialog(int option)
 {
     QColorDialog *dialog = nullptr;
     auto tool = SpectacleCore::instance()->annotationDocument()->tool();
-    auto wrapper = SpectacleCore::instance()->annotationDocument()->selectedItemWrapper();
+    auto saWrapper = SpectacleCore::instance()->annotationDocument()->selectedActionWrapper();
 
     std::function<QColor()> toolGetter;
-    std::function<QColor()> wrapperGetter;
+    std::function<QColor()> sawGetter;
     std::function<void(const QColor &)> toolSetter;
-    std::function<void(const QColor &)> wrapperSetter;
+    std::function<void(const QColor &)> sawSetter;
     using namespace std::placeholders; // for std::placeholders::_1
-    if (option == AnnotationTool::StrokeOption) {
+    if (option == AnnotationTool::Stroke) {
         toolGetter = std::bind(&AnnotationTool::strokeColor, tool);
-        wrapperGetter = std::bind(&SelectedItemWrapper::strokeColor, wrapper);
+        sawGetter = std::bind(&SelectedActionWrapper::strokeColor, saWrapper);
         toolSetter = std::bind(&AnnotationTool::setStrokeColor, tool, _1);
-        wrapperSetter = std::bind(&SelectedItemWrapper::setStrokeColor, wrapper, _1);
-    } else if (option == AnnotationTool::FillOption) {
+        sawSetter = std::bind(&SelectedActionWrapper::setStrokeColor, saWrapper, _1);
+    } else if (option == AnnotationTool::Fill) {
         toolGetter = std::bind(&AnnotationTool::fillColor, tool);
-        wrapperGetter = std::bind(&SelectedItemWrapper::fillColor, wrapper);
+        sawGetter = std::bind(&SelectedActionWrapper::fillColor, saWrapper);
         toolSetter = std::bind(&AnnotationTool::setFillColor, tool, _1);
-        wrapperSetter = std::bind(&SelectedItemWrapper::setFillColor, wrapper, _1);
-    } else if (option == AnnotationTool::FontOption) {
+        sawSetter = std::bind(&SelectedActionWrapper::setFillColor, saWrapper, _1);
+    } else if (option == AnnotationTool::Font) {
         toolGetter = std::bind(&AnnotationTool::fontColor, tool);
-        wrapperGetter = std::bind(&SelectedItemWrapper::fontColor, wrapper);
+        sawGetter = std::bind(&SelectedActionWrapper::fontColor, saWrapper);
         toolSetter = std::bind(&AnnotationTool::setFontColor, tool, _1);
-        wrapperSetter = std::bind(&SelectedItemWrapper::setFontColor, wrapper, _1);
+        sawSetter = std::bind(&SelectedActionWrapper::setFontColor, saWrapper, _1);
     } else {
         qmlWarning(this) << "invalid option argument";
         return;
     }
 
     QColor color;
-    if (tool->type() == AnnotationTool::SelectTool
-        || (tool->type() == AnnotationTool::TextTool
-            && wrapper->options().testFlag(AnnotationTool::TextOption))
+    if (tool->type() == AnnotationDocument::ChangeAction
+        || (tool->type() == AnnotationDocument::Text
+            && saWrapper->type() == AnnotationDocument::Text)
     ) {
-        color = wrapperGetter();
+        color = sawGetter();
     } else {
         color = toolGetter();
     }
@@ -377,24 +409,26 @@ void SpectacleWindow::showColorDialog(int option)
     dialog->setAttribute(Qt::WA_DeleteOnClose);
     dialog->setOption(QColorDialog::ShowAlphaChannel);
 
-    setWidgetTransientParent(dialog, this);
+    if (dialog->winId()) {
+        dialog->windowHandle()->setTransientParent(this);
+    }
 
     if (flags().testFlag(Qt::WindowStaysOnTopHint)) {
         dialog->setWindowFlag(Qt::WindowStaysOnTopHint);
     }
 
-    connect(dialog, &QColorDialog::colorSelected, this, [toolSetter, wrapperSetter](const QColor &color){
+    connect(dialog, &QColorDialog::colorSelected, this, [toolSetter, sawSetter](const QColor &color){
         auto tool = SpectacleCore::instance()->annotationDocument()->tool();
-        auto wrapper = SpectacleCore::instance()->annotationDocument()->selectedItemWrapper();
-        if (tool->type() == AnnotationTool::SelectTool) {
-            wrapperSetter(color);
-            wrapper->commitChanges();
-        } else if (tool->type() == AnnotationTool::TextTool
-            && wrapper->options().testFlag(AnnotationTool::TextOption)
+        auto saw = SpectacleCore::instance()->annotationDocument()->selectedActionWrapper();
+        if (tool->type() == AnnotationDocument::ChangeAction) {
+            sawSetter(color);
+            saw->commitChanges();
+        } else if (tool->type() == AnnotationDocument::Text
+            && saw->type() == AnnotationDocument::Text
         ) {
             toolSetter(color);
-            wrapperSetter(color);
-            wrapper->commitChanges();
+            sawSetter(color);
+            saw->commitChanges();
         } else {
             toolSetter(color);
         }
@@ -403,6 +437,60 @@ void SpectacleWindow::showColorDialog(int option)
     dialog->open();
 }
 
+void SpectacleWindow::openUrlExternally(const QUrl &url)
+{
+    auto job = new KIO::OpenUrlJob(url);
+    job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, nullptr));
+    job->start();
+}
+
+void SpectacleWindow::openContainingFolder(const QUrl &url)
+{
+    KIO::highlightInFileManager({url});
+}
+
+void SpectacleWindow::startDrag()
+{
+    auto exportManager = ExportManager::instance();
+    if (exportManager->pixmap().isNull()) {
+        return;
+    }
+
+    const QUrl tempFile = SpectacleCore::instance()->videoMode() ? SpectacleCore::instance()->currentVideo() : exportManager->tempSave();
+    if (!tempFile.isValid()) {
+        return;
+    }
+
+    auto mimeData = new QMimeData;
+    mimeData->setUrls(QList<QUrl>{tempFile});
+    // "application/x-kde-suggestedfilename" is handled by KIO/PasteJob.
+    // It is only used when QMimeData::formats() is empty or when the user is
+    // prompted to set a filename for the content after drag & drop or paste.
+    // When QMimeData::formats() is empty, a dialog for picking the data format
+    // is supposed to appear.
+    // It's likely that users will never see the data format dialog with Spectacle.
+    mimeData->setData(QStringLiteral("application/x-kde-suggestedfilename"), QFile::encodeName(tempFile.fileName()));
+    KUrlMimeData::exportUrlsToPortal(mimeData);
+
+    auto dragHandler = new QDrag(this);
+    dragHandler->setMimeData(mimeData);
+
+    if (SpectacleCore::instance()->videoMode()) {
+        Kirigami::Units units;
+        auto iconSize = units.iconSizes()->large();
+        dragHandler->setPixmap(QIcon::fromTheme(QStringLiteral("video-x-matroska")).pixmap(iconSize, iconSize));
+    } else {
+        QSize size = exportManager->pixmap().size();
+        // TODO: use the composed pixmap with annotations instead
+        if (size.width() > 256 || size.height() > 256) {
+            dragHandler->setPixmap(exportManager->pixmap().scaled(256, 256, Qt::KeepAspectRatio, Qt::SmoothTransformation));
+        } else {
+            dragHandler->setPixmap(exportManager->pixmap());
+        }
+    }
+    dragHandler->exec(Qt::CopyAction);
+}
+
 void SpectacleWindow::mousePressEvent(QMouseEvent *event)
 {
     // QMenus need to be closed by hand when used from QML, see plasma-workspace/shellcorona.cpp
@@ -421,7 +509,18 @@ void SpectacleWindow::keyPressEvent(QKeyEvent *event)
     if (event->isAccepted()) {
         return;
     }
-    m_pressedKeys = event->key() | event->modifiers();
+    // Cancel defaults to Escape in QPlatformTheme
+    if (event->matches(QKeySequence::Quit)
+        || event->matches(QKeySequence::Close)
+        || event->matches(QKeySequence::Cancel)) {
+        // we must do the shortcut here to prevent closing dialogs from closing Spectacle
+        auto spectacleCore = SpectacleCore::instance();
+        if (spectacleCore->captureTimeRemaining() > 0) {
+            spectacleCore->cancelScreenshot();
+        } else {
+            Q_EMIT engine()->quit();
+        }
+    }
 }
 
 void SpectacleWindow::keyReleaseEvent(QKeyEvent *event)
@@ -431,18 +530,10 @@ void SpectacleWindow::keyReleaseEvent(QKeyEvent *event)
     if (event->isAccepted()) {
         return;
     }
-    // Cancel defaults to Escape in QPlatformTheme.
-    // Handling this here fixes https://bugs.kde.org/show_bug.cgi?id=428478
-    if ((event->matches(QKeySequence::Quit)
+    if (event->matches(QKeySequence::Quit)
         || event->matches(QKeySequence::Close)
-        || event->matches(QKeySequence::Cancel))
-        // We need to check if these were pressed previously or else pressing escape
-        // in a dialog will quit spectacle when you release the escape key.
-        && m_pressedKeys == (event->key() | event->modifiers())
-    ) {
+        || event->matches(QKeySequence::Cancel)) {
         event->accept();
-        auto spectacleCore = SpectacleCore::instance();
-        spectacleCore->cancelScreenshot();
     } else if (event->matches(QKeySequence::Preferences)) {
         event->accept();
         showPreferencesDialog();
@@ -451,17 +542,6 @@ void SpectacleWindow::keyReleaseEvent(QKeyEvent *event)
         SpectacleCore::instance()->takeNewScreenshot();
     } else if (event->matches(QKeySequence::HelpContents)) {
         event->accept();
-        HelpMenu::instance()->showAppHelp();
+        m_helpMenu->showAppHelp();
     }
-    m_pressedKeys = {};
 }
-
-bool SpectacleWindow::event(QEvent *event)
-{
-    if (event->type() == QEvent::DevicePixelRatioChange) {
-        Q_EMIT devicePixelRatioChanged();
-    }
-    return QQuickView::event(event);
-}
-
-#include "moc_SpectacleWindow.cpp"
