@@ -6,17 +6,17 @@
 
 #include "PlatformKWinWayland.h"
 
-#include <qplatformdefs.h>
-#include <QtConcurrent>
 #include <QApplication>
+#include <QDBusInterface>
+#include <QDBusPendingCall>
+#include <QDBusUnixFileDescriptor>
+#include <QFutureWatcher>
+#include <QGuiApplication>
 #include <QImage>
 #include <QPixmap>
-#include <QDBusInterface>
-#include <QDBusUnixFileDescriptor>
-#include <QDBusPendingCall>
-#include <QFutureWatcher>
 #include <QScreen>
-#include <QGuiApplication>
+#include <QtConcurrent>
+#include <qplatformdefs.h>
 
 #include <array>
 
@@ -86,7 +86,7 @@ static QVector<QImage> readImages(int thePipeFd)
 
     QImage lImage;
     QVector<QImage> imgs;
-    while (!lDataStream.atEnd()){
+    while (!lDataStream.atEnd()) {
         lDataStream >> lImage;
         if (!lImage.isNull()) {
             imgs << lImage;
@@ -98,35 +98,39 @@ static QVector<QImage> readImages(int thePipeFd)
 
 /* -- General Plumbing ------------------------------------------------------------------------- */
 
-PlatformKWinWayland::PlatformKWinWayland(QObject *parent) :
-    Platform(parent)
-{}
-
-QString PlatformKWinWayland::platformName() const
+PlatformKWinWayland::PlatformKWinWayland(QObject *parent)
+    : Platform(parent)
 {
-    return QStringLiteral("KWinWayland");
+    updateSupportedGrabModes();
+    connect(qGuiApp, &QGuiApplication::screenAdded, this, &PlatformKWinWayland::updateSupportedGrabModes);
+    connect(qGuiApp, &QGuiApplication::screenRemoved, this, &PlatformKWinWayland::updateSupportedGrabModes);
 }
 
 static std::array<int, 3> s_plasmaVersion = {-1, -1, -1};
 
-std::array<int, 3> findPlasmaMinorVersion () {
+std::array<int, 3> findPlasmaMinorVersion()
+{
     if (s_plasmaVersion == std::array<int, 3>{-1, -1, -1}) {
         auto message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.plasmashell"),
-                                       QStringLiteral("/MainApplication"),
-                                       QStringLiteral("org.freedesktop.DBus.Properties"),
-                                       QStringLiteral("Get"));
+                                                      QStringLiteral("/MainApplication"),
+                                                      QStringLiteral("org.freedesktop.DBus.Properties"),
+                                                      QStringLiteral("Get"));
 
         message.setArguments({QStringLiteral("org.qtproject.Qt.QCoreApplication"), QStringLiteral("applicationVersion")});
 
         const auto resultMessage = QDBusConnection::sessionBus().call(message);
         if (resultMessage.type() != QDBusMessage::ReplyMessage) {
-            qWarning() << "Error querying plasma version" << resultMessage.errorName() << resultMessage .errorMessage();
+            qWarning() << "Error querying plasma version" << resultMessage.errorName() << resultMessage.errorMessage();
             return s_plasmaVersion;
         }
         QDBusVariant val = resultMessage.arguments().at(0).value<QDBusVariant>();
 
         const QString rawVersion = val.variant().value<QString>();
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         const QVector<QStringRef> splitted = rawVersion.splitRef(QLatin1Char('.'));
+#else
+        const QVector<QStringView> splitted = QStringView(rawVersion).split(QLatin1Char('.'));
+#endif
         if (splitted.size() != 3) {
             qWarning() << "error parsing plasma version";
             return s_plasmaVersion;
@@ -154,26 +158,38 @@ std::array<int, 3> findPlasmaMinorVersion () {
 
 Platform::GrabModes PlatformKWinWayland::supportedGrabModes() const
 {
-    Platform::GrabModes lSupportedModes({ Platform::GrabMode::AllScreens, GrabMode::WindowUnderCursor });
+    return m_grabModes;
+}
+
+void PlatformKWinWayland::updateSupportedGrabModes()
+{
+    GrabModes grabModes = {Platform::GrabMode::AllScreens, GrabMode::WindowUnderCursor};
     QList<QScreen *> screens = QApplication::screens();
+    int screenCount = screens.count();
 
     // TODO remove sometime after Plasma 5.21 is released
     // We can handle rectangular selection one one screen not scale factor
     // on Plasma < 5.21
-    if (screenshotScreensAvailable() || (screens.count() == 1 && screens.first()->devicePixelRatio() == 1)) {
-        lSupportedModes |= Platform::GrabMode::PerScreenImageNative;
+    if (screenshotScreensAvailable()
+        || (screenCount == 1 && screens.first()->devicePixelRatio() == 1)) {
+        grabModes |= Platform::GrabMode::PerScreenImageNative;
     }
 
-    // TODO remove sometime after Plasma 5.20 is released
-    auto plasmaVersion = findPlasmaMinorVersion();
-    if (plasmaVersion.at(0) != -1 && (plasmaVersion.at(0) != 5 || (plasmaVersion.at(1) >= 20))) {
-        lSupportedModes |= Platform::GrabMode::AllScreensScaled;
+    if (screenCount > 1) {
+        grabModes |= Platform::GrabMode::CurrentScreen;
+
+        // TODO remove sometime after Plasma 5.20 is released
+        auto plasmaVersion = findPlasmaMinorVersion();
+        if (plasmaVersion.at(0) != -1
+            && (plasmaVersion.at(0) != 5 || (plasmaVersion.at(1) >= 20))) {
+            grabModes |= Platform::GrabMode::AllScreensScaled;
+        }
     }
 
-    if (screens.count() > 1) {
-        lSupportedModes |= Platform::GrabMode::CurrentScreen;
+    if (m_grabModes != grabModes) {
+        m_grabModes = grabModes;
+        Q_EMIT supportedGrabModesChanged();
     }
-    return lSupportedModes;
 }
 
 bool PlatformKWinWayland::screenshotScreensAvailable() const
@@ -193,15 +209,15 @@ Platform::ShutterModes PlatformKWinWayland::supportedShutterModes() const
     // TODO remove sometime after Plasma 5.20 is released
     auto plasmaVersion = findPlasmaMinorVersion();
     if (plasmaVersion.at(0) != -1 && (plasmaVersion.at(0) != 5 || (plasmaVersion.at(1) >= 20))) {
-        return { ShutterMode::Immediate };
+        return {ShutterMode::Immediate};
     } else {
-        return { ShutterMode::OnClick };
+        return {ShutterMode::OnClick};
     }
 }
 
 void PlatformKWinWayland::doGrab(ShutterMode /* theShutterMode */, GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations)
 {
-    switch(theGrabMode) {
+    switch (theGrabMode) {
     case GrabMode::AllScreens:
         doGrabHelper(QStringLiteral("screenshotFullscreen"), theIncludePointer, true);
         return;
@@ -209,8 +225,7 @@ void PlatformKWinWayland::doGrab(ShutterMode /* theShutterMode */, GrabMode theG
         doGrabHelper(QStringLiteral("screenshotFullscreen"), theIncludePointer, false);
         return;
 
-    case GrabMode::PerScreenImageNative:
-    {
+    case GrabMode::PerScreenImageNative: {
         const QList<QScreen *> screens = QGuiApplication::screens();
         QStringList screenNames;
         screenNames.reserve(screens.count());
@@ -230,6 +245,8 @@ void PlatformKWinWayland::doGrab(ShutterMode /* theShutterMode */, GrabMode theG
         doGrabHelper(QStringLiteral("screenshotScreen"), theIncludePointer);
         return;
     }
+    case GrabMode::ActiveWindow:
+    case GrabMode::TransientWithParent:
     case GrabMode::WindowUnderCursor: {
         int lOpMask = theIncludeDecorations ? 1 : 0;
         if (theIncludePointer) {
@@ -238,10 +255,8 @@ void PlatformKWinWayland::doGrab(ShutterMode /* theShutterMode */, GrabMode theG
         doGrabHelper(QStringLiteral("interactive"), lOpMask);
         return;
     }
-    case GrabMode::InvalidChoice:
-    case GrabMode::ActiveWindow:
-    case GrabMode::TransientWithParent:
-        emit newScreenshotFailed();
+    case GrabMode::NoGrabModes:
+        Q_EMIT newScreenshotFailed();
         return;
     }
 }
@@ -251,39 +266,45 @@ void PlatformKWinWayland::doGrab(ShutterMode /* theShutterMode */, GrabMode theG
 void PlatformKWinWayland::startReadImage(int theReadPipe)
 {
     auto lWatcher = new QFutureWatcher<QImage>(this);
-    QObject::connect(lWatcher, &QFutureWatcher<QImage>::finished, this,
-        [lWatcher, this] () {
-            lWatcher->deleteLater();
-            const QImage lImage = lWatcher->result();
-            if (lImage.isNull()) {
-                Q_EMIT newScreenshotFailed();
-            } else {
-                Q_EMIT newScreenshotTaken(QPixmap::fromImage(lImage));
-            }
+    QObject::connect(lWatcher, &QFutureWatcher<QImage>::finished, this, [lWatcher, this]() {
+        lWatcher->deleteLater();
+        const QImage lImage = lWatcher->result();
+        if (lImage.isNull()) {
+            Q_EMIT newScreenshotFailed();
+        } else {
+            Q_EMIT newScreenshotTaken(QPixmap::fromImage(lImage));
         }
-    );
+    });
     lWatcher->setFuture(QtConcurrent::run(readImage, theReadPipe));
 }
 
 void PlatformKWinWayland::startReadImages(int theReadPipe)
 {
     auto lWatcher = new QFutureWatcher<QVector<QImage>>(this);
-    QObject::connect(lWatcher, &QFutureWatcher<QVector<QImage>>::finished, this,
-        [lWatcher, this] () {
-            lWatcher->deleteLater();
-            auto result = lWatcher->result();
-            if (result.isEmpty()) {
+    QObject::connect(lWatcher, &QFutureWatcher<QVector<QImage>>::finished, this, [lWatcher, this]() {
+        lWatcher->deleteLater();
+        auto images = lWatcher->result();
+        if (images.isEmpty()) {
+            Q_EMIT newScreenshotFailed();
+        } else {
+            QVector<ScreenImage> screenImages;
+            const auto &screens = qGuiApp->screens();
+            if (images.length() != screens.length()) {
+                qWarning() << "ERROR: number of screens does not match number of images, expected:" << images.length() << "actual:" << screens.length();
                 Q_EMIT newScreenshotFailed();
-            } else {
-                Q_EMIT newScreensScreenshotTaken(result);
+                return;
             }
+            for (int i = 0; i < screens.length(); ++i) {
+                screenImages.append({screens.at(i), images.at(i), static_cast<qreal>(images.at(i).width()) / screens.at(i)->geometry().width()});
+            }
+            Q_EMIT newScreensScreenshotTaken(screenImages);
         }
-    );
+    });
     lWatcher->setFuture(QtConcurrent::run(readImages, theReadPipe));
 }
 
-template <typename ... ArgType>
-void PlatformKWinWayland::callDBus(const QString &theGrabMethod, int theWriteFile, ArgType ... arguments)
+template<typename... ArgType>
+void PlatformKWinWayland::callDBus(const QString &theGrabMethod, int theWriteFile, ArgType... arguments)
 {
     QDBusInterface lInterface(QStringLiteral("org.kde.KWin"), QStringLiteral("/Screenshot"), QStringLiteral("org.kde.kwin.Screenshot"));
     QDBusPendingCall pcall = lInterface.asyncCall(theGrabMethod, QVariant::fromValue(QDBusUnixFileDescriptor(theWriteFile)), arguments...);
@@ -293,8 +314,7 @@ void PlatformKWinWayland::callDBus(const QString &theGrabMethod, int theWriteFil
 void PlatformKWinWayland::checkDbusPendingCall(const QDBusPendingCall &pcall)
 {
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pcall, this);
-    QObject::connect(watcher, &QDBusPendingCallWatcher::finished,
-                     this, [this](QDBusPendingCallWatcher* watcher) {
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *watcher) {
         if (watcher->isError()) {
             const auto error = watcher->error();
             qWarning() << "Error calling KWin DBus interface:" << error.name() << error.message();
@@ -304,12 +324,12 @@ void PlatformKWinWayland::checkDbusPendingCall(const QDBusPendingCall &pcall)
     });
 }
 
-template <typename ... ArgType>
-void PlatformKWinWayland::doGrabHelper(const QString &theGrabMethod, ArgType ... arguments)
+template<typename... ArgType>
+void PlatformKWinWayland::doGrabHelper(const QString &theGrabMethod, ArgType... arguments)
 {
     int lPipeFds[2];
-    if (pipe2(lPipeFds, O_CLOEXEC|O_NONBLOCK) != 0) {
-        emit newScreenshotFailed();
+    if (pipe2(lPipeFds, O_CLOEXEC | O_NONBLOCK) != 0) {
+        Q_EMIT newScreenshotFailed();
         return;
     }
 
@@ -319,12 +339,12 @@ void PlatformKWinWayland::doGrabHelper(const QString &theGrabMethod, ArgType ...
     close(lPipeFds[1]);
 }
 
-template <typename ... ArgType>
-void PlatformKWinWayland::doGrabImagesHelper(const QString &theGrabMethod, ArgType ... arguments)
+template<typename... ArgType>
+void PlatformKWinWayland::doGrabImagesHelper(const QString &theGrabMethod, ArgType... arguments)
 {
     int lPipeFds[2];
-    if (pipe2(lPipeFds, O_CLOEXEC|O_NONBLOCK) != 0) {
-        emit newScreenshotFailed();
+    if (pipe2(lPipeFds, O_CLOEXEC | O_NONBLOCK) != 0) {
+        Q_EMIT newScreenshotFailed();
         return;
     }
 
