@@ -10,6 +10,7 @@
 #include "PlatformKWinWayland.h"
 #include "PlatformKWinWayland2.h"
 #include "PlatformNull.h"
+#include "VideoPlatformWayland.h"
 
 #ifdef XCB_FOUND
 #include "PlatformXcb.h"
@@ -22,7 +23,7 @@ PlatformPtr loadPlatform()
     // We might be using the XCB platform (with Xwayland) in a wayland session,
     // but the X11 grabber won't work in that case. So force the Wayland grabber
     // in Wayland sessions.
-    if (KWindowSystem::isPlatformWayland() || qstrcmp(qgetenv("XDG_SESSION_TYPE").data(), "wayland") == 0) {
+    if (KWindowSystem::isPlatformWayland() || qstrcmp(qgetenv("XDG_SESSION_TYPE").constData(), "wayland") == 0) {
         std::unique_ptr<Platform> platform = PlatformKWinWayland2::create();
         if (platform) {
             return platform;
@@ -40,3 +41,11 @@ PlatformPtr loadPlatform()
     // If nothing else worked, return the null platform
     return std::make_unique<PlatformNull>();
 }
+
+VideoPlatformPtr loadVideoPlatform()
+{
+    if (KWindowSystem::isPlatformWayland()) {
+        return std::make_unique<VideoPlatformWayland>();
+    }
+    return std::make_unique<VideoPlatformNull>();
+}
