@@ -292,10 +292,9 @@ qreal SpectacleCore::captureProgress() const
 
 void SpectacleCore::activate(const QStringList &arguments, const QString &workingDirectory)
 {
-    Q_UNUSED(workingDirectory)
-
-    // Remove any windows if they are present.
-    deleteWindows();
+    if (!workingDirectory.isEmpty()) {
+        QDir::setCurrent(workingDirectory);
+    }
 
     // We can't re-use QCommandLineParser instances, it preserves earlier parsed values
     QCommandLineParser parser;
@@ -335,6 +334,13 @@ void SpectacleCore::activate(const QStringList &arguments, const QString &workin
         }
     }
 
+    if (parser.optionNames().size() > 0 || m_startMode != StartMode::Gui) {
+        // Delete windows if we have CLI options or not in GUI mode.
+        // We don't want to delete them otherwise because that will mess with the
+        // settings for PrintScreen key behavior.
+        deleteWindows();
+    }
+
     /* The logic for setting options for each start mode:
      *
      * - Gui/DBus: Prioritise command line options and default to saved settings.
@@ -813,7 +819,7 @@ void SpectacleCore::syncExportPixmap()
     for (auto &img : m_annotationDocument->baseImages()) {
         maxDpr = qMax(maxDpr, img.devicePixelRatio());
     }
-    QRectF imageRect(QPointF(0, 0), m_annotationDocument->canvasSize());
+    QRectF imageRect(QPointF(0, 0), m_annotationDocument->canvasSize() * maxDpr);
     const auto &image = m_annotationDocument->renderToImage(imageRect, maxDpr);
     ExportManager::instance()->setPixmap(QPixmap::fromImage(image));
 }
